clear all; clc; close all;

% PARAMETERS
% A=1; % gaussian amplitude
b=-pi/2; % chirp
p=-1e4;
N=2^16;
z=100; % distance in m
% lambda0=0.8; % in um (800nm)
lambda0=1.55; % in um (1550nm)

% t=linspace(-10e-13,10e-13,N); 
t=linspace(0,1e-10,N);
dt=(t(2)-t(1));
df=1/dt;
% f=((0:N-1)-N/2)/df;
f=[-N/2:(N-1)/2]*df/N;
w=2*pi*f;
% f0=3e8/800e-9; % f=3.75e14 Hz
f0=3e8/1.55e-6; % f=1.935e14 Hz
w0=2*pi*f0;
c=3e14; %in um/s

A=[50];
for k=1:length(A)
% Signal before propagation
% FWHM=20e-15;
FWHM=2e-12;
sigma=FWHM/(2*sqrt(2*log(2)));
gaussian=A(k)*exp(-((t-t(N/2)).^2)/sigma^2);
% Et=gaussian.*exp(j*w0*t).*exp(j*b/sigma^2*t.^2);
Et=gaussian.*exp(j*w0*t).*exp(j*p*(sawtooth(f/1e14)+1));

% Spectrum before propagation
Ef=real(fft(Et))/sqrt(N)/2; 

med=max(Ef)/2;
width=find(Ef>med);
Af=f(max(width))-f(min(width));

%Ploting the spectrogram, signal and spectrum
figure,subplot(2,2,1),text(0.5,0.5,'Before propagating','FontSize',14,'HorizontalAlignment','center'),axis off
text(0.5,0.2,['A=',num2str(A(k)),'; Af=',num2str(Af,3)],'FontSize',14,'HorizontalAlignment','center'),
subplot(2,2,2),plot(t,abs(Et)),axis ([min(t) max(t) min(abs(Et)) max(abs(Et))])
title('Signal in Time'),xlabel('Time (s)'),ylabel('Amplitude (V)')
subplot(2,2,4),sgram(Et,2*max(f),40),colorbar('hide'),title('Spectrogram'),,axis ([0 max(t) 1.3e14 1.9e14])
subplot(2,2,3),plot(f,abs(fftshift(Ef))),axis ([1.3e14 1.9e14 0 max(real(Ef))]),view(-90,90)
title('Spectrum'),ylabel('Amplitude (V)')

% % Linear Propagation
% % fused silica fiber
% a='fused silica fiber';
% n=1.453;
% dn=-0.017;  % um^-2
% dn2=0.040;  % um^-2
% dn3=-0.239; % um^-3
% % % LakL21
% % a='LakL21';
% % n=1.632;
% % dn=-0.026;  % um^-2
% % dn2=0.066;  % um^-2
% % dn3=-0.386; % um^-3
% % % SF10
% % a='SF10';
% % n=1.711;
% % dn=-0.050;  % um^-2
% % dn2=0.176;  % um^-2
% % dn3=-0.988; % um^-3
% % % BK7
% % a='BK7';
% % n=1.511;
% % dn=-0.015;  % um^-2
% % dn2=0.056;  % um^-2
% % dn3=-0.288; % um^-3
% 
% k0=2*pi/lambda0; %um^-1 
% dk=(n-(lambda0*dn))/c; % 1/vg (s/um)
% dk2=(lambda0^3)*dn2/(2*pi*(c^2)); % GVD (s^2/um)
% dk3=-(1/(4*(pi^2)*(c^3)))*((3*(lambda0^4)*dn2)+(lambda0^5)*dn3); % TOD (s^3/um)
% k=k0+dk*(w-w0)+0.5*dk2*(w-w0).^2+1/6*dk3*(w-w0).^3; % in um^-1
% 
% Efz=fft(Et,N).*exp(j*k*z)/sqrt(N)/2; %deleted *1e6
% Etz=ifft(Efz,N)*sqrt(N)*2;

% Nonlinear Propagation
a='Nonlinear Propagation';
delta_z = 1e-2;
n = ceil(z/delta_z);      
dz = z/n;	
T = N*(t(2)-t(1));				% T(time period = N * dt
omega = 2*pi/T*[(0:N/2-1) (-N/2:-1)];	% angular freq. axis

beta_2 = 5.1*1e-27;                        % s^2/km
gamma = 2.2e-3;                            % 1/W km
alpha = 0.051e-3;                          % 1/km

D = 1/2*(sqrt(-1)*beta_2*omega.^2 - alpha)*delta_z;
exp_D = exp(D);

Etz = ifft(exp(0.5*D).*fft(Et));	% The first half step
for i = 1:n
   Etz = ifft(exp_D.*fft(Etz.*exp(delta_z*sqrt(-1)*gamma*abs(Etz).^2)));
end
Etz = ifft(exp(-0.5*D).*fft(Etz));  % The last half step backwards

% small code to center the gaussian in 0
aux=Etz;
center=find(Etz==max(Etz));
num=N-center;
if center>N/2
        Etz(N/2:num+N/2)=aux(center:N);
        Etz(num+N/2+1:N)=aux(1:N/2-num);
        Etz(1:N/2-1)=aux(N/2-num+1:center-1);
elseif center<N/2
        Etz(N/2:N)=aux(center:N/2+center);
        Etz(1:num-N/2)=aux(N/2+center+1:N);
        Etz(num-N/2+1:N/2-1)=aux(1:center-1);
end
    
Efz=real(fft(Etz))/sqrt(N)/2; 

medz=max(Efz)/2;
widthz=find(Efz>medz);
Afz=f(max(widthz))-f(min(widthz));

%Ploting the spectrogram, signal and spectrum
figure,
subplot(2,2,1),text(0.5,0.5,['Propagating: ',a,' z=',num2str(z),'m'],'FontSize',14,'HorizontalAlignment','center'),axis off
text(0.5,0.2,['A=',num2str(A(k)),'; Af=',num2str(Afz,3)],'FontSize',14,'HorizontalAlignment','center'),
subplot(2,2,2),plot(t,abs(Etz)),axis ([min(t) max(t) min(abs(Et)) max(abs(Et))])
title('Signal in Time'),xlabel('Time (s)'),ylabel('Amplitude (V)')
subplot(2,2,4),sgram(Etz,2*max(f),40),colorbar('hide'),title('Spectrogram'),axis ([0 max(t) 1.3e14 1.9e14])
subplot(2,2,3),plot(f,abs(fftshift(Efz))),axis ([1.3e14 1.9e14 0 max(real(Ef))]),view(-90,90)
title('Spectrum'),ylabel('Amplitude (V)')
end
% figure,subplot(1,2,1),plot(t,abs(Et)),hold on, plot(t,(b/sigma^2*t.^2)/p,'r'),hold off
% title('Previous experiment'),xlabel('Time (s)'),ylabel('Amplitude (V)')
% subplot(1,2,2),plot(t,abs(Et)),hold on, plot(t,(sawtooth(f/1e14)+1),'r'),hold off
% title('Sawtooth phase'),xlabel('Time (s)'),ylabel('Amplitude (V)')