clear all; clc; close all;

% PARAMETERS
A=2; % gaussian amplitude
b=pi/2; % chirp
% p=1e4;
FWHM=2e-12;
N=2^16;
z=100; % distance in m
lambda0=1.55; % in um (1550nm)
t=linspace(0,1e-10,N);

dt=(t(2)-t(1));
df=1/dt;
% f=((0:N-1)-N/2)/df;
f=[-N/2:(N-1)/2]*df/N;
w=2*pi*f;
f0=3e8/1.55e-6; % f=1.935e14 Hz
w0=2*pi*f0;
c=3e14; %in um/s

% Signal before propagation
sigma=FWHM/(2*sqrt(2*log(2)));
gaussian=A*exp(-((t-t(N/2)).^2)/sigma^2);
Et=gaussian.*exp(j*w0*t).*exp(j*b/sigma^2*t.^2);
% Et=gaussian.*exp(-j*w0*t).*exp(-j*p*sawtooth(f/100));

% Spectrum before propagation
Ef=real(fft(Et))/sqrt(N)/2; 

% Code to get the pulse width
med=max(Ef)/2;
width=find(Ef>med);
Af=f(max(width))-f(min(width));

% Ploting the spectrogram, signal and spectrum
figure,subplot(2,2,1)
text(0.5,0.5,'Before propagating','FontSize',14,'HorizontalAlignment','center'),axis off
text(0.5,0.2,['A=',num2str(A),'; b=',num2str(b),'; Af=',num2str(Af,4),'Hz'],'FontSize',14,'HorizontalAlignment','center'),
subplot(2,2,2),plot(t,abs(Et))
axis ([min(t) max(t) min(abs(Et)) max(abs(Et))])
title('Signal in Time'),xlabel('Time (s)'),ylabel('Amplitude (V)')
subplot(2,2,4),sgram(Et,2*max(f),40),colorbar('hide')
title('Spectrogram'),,axis ([0 max(t) 0 max(f)])
subplot(2,2,3),plot(f,abs(fftshift(Ef))),axis ([0 max(f) 0 max(real(Ef))])
view(-90,90),title('Spectrum'),ylabel('Amplitude (V)')

% Nonlinear Propagation
a='Nonlinear Propagation';
delta_z = 1e-2;
n = ceil(z/delta_z);      
dz = z/n;	
T = N*(t(2)-t(1));				% T(time period = N * dt
omega = 2*pi/T*[(0:N/2-1) (-N/2:-1)];	% angular freq. axis

beta_2 = 5.1*1e-27;                        % s^2/km
gamma = 2.2e-3;                            % 1/W km
alpha = 0.051e-3;                          % 1/km

D = 1/2*(sqrt(-1)*beta_2*omega.^2 - alpha)*delta_z;
exp_D = exp(D);

Etz = ifft(exp(0.5*D).*fft(Et));	% The first half step
for i = 1:n
   Etz = ifft(exp_D.*fft(Etz.*exp(delta_z*sqrt(-1)*gamma*abs(Etz).^2)));
end
Etz = ifft(exp(-0.5*D).*fft(Etz));  % The last half step backwards

% code to center the gaussian in 0
aux=Etz;
center=find(Etz==max(Etz));
num=N-center;
if center>N/2
        Etz(N/2:num+N/2)=aux(center:N);
        Etz(num+N/2+1:N)=aux(1:N/2-num);
        Etz(1:N/2-1)=aux(N/2-num+1:center-1);
elseif center<N/2
        Etz(N/2:N)=aux(center:N/2+center);
        Etz(1:num-N/2)=aux(N/2+center+1:N);
        Etz(num-N/2+1:N/2-1)=aux(1:center-1);
end
    
Efz=real(fft(Etz))/sqrt(N)/2; 

% Code to get the pulse width
medz=max(Efz)/2;
widthz=find(Efz>medz);
Afz=f(max(widthz))-f(min(widthz));

%Ploting the spectrogram, signal and spectrum
figure,subplot(2,2,1)
text(0.5,0.5,['Propagating: ',a,' z=',num2str(z),'m'],'FontSize',14,'HorizontalAlignment','center'),axis off
text(0.5,0.2,['A=',num2str(A),'; b=',num2str(b),'; Af=',num2str(Afz,4),'Hz'],'FontSize',14,'HorizontalAlignment','center'),
subplot(2,2,2),plot(t,abs(Etz))
axis ([min(t) max(t) min(abs(Et)) max(abs(Et))])
title('Signal in Time'),xlabel('Time (s)'),ylabel('Amplitude (V)')
subplot(2,2,4),sgram(Etz,2*max(f),40)
colorbar('hide'),title('Spectrogram'),axis ([0 max(t) 0 max(f)])
subplot(2,2,3),plot(f,abs(fftshift(Efz)))
axis ([0 max(f) 0 max(real(Ef))]),view(-90,90)
title('Spectrum'),ylabel('Amplitude (V)')